--- old/grub/grub-core/fs/zfs/zfsinfo.c	2012-08-31 05:08:54.303066896 +0400
+++ new/grub/grub-core/fs/zfs/zfsinfo.c	2012-08-31 05:08:54.339887022 +0400
@@ -184,9 +184,10 @@
 }
 
 static grub_err_t
-get_bootpath (char *nvlist, char **bootpath, char **devid)
+get_bootpath (char *nvlist, char **bootpath, char **devid, grub_uint64_t inguid)
 {
   char *type = 0;
+  grub_uint64_t diskguid = 0;
 
   type = grub_zfs_nvlist_lookup_string (nvlist, ZPOOL_CONFIG_TYPE);
 
@@ -195,10 +196,11 @@
 
   if (grub_strcmp (type, VDEV_TYPE_DISK) == 0)
     {
+      grub_zfs_nvlist_lookup_uint64 (nvlist, "guid", &diskguid);
       *bootpath = grub_zfs_nvlist_lookup_string (nvlist,
 						 ZPOOL_CONFIG_PHYS_PATH);
       *devid = grub_zfs_nvlist_lookup_string (nvlist, ZPOOL_CONFIG_DEVID);
-      if (!*bootpath || !*devid)
+      if (!*bootpath || !*devid || (diskguid != inguid))
 	{
 	  grub_free (*bootpath);
 	  grub_free (*devid);
@@ -223,7 +225,7 @@
 						       ZPOOL_CONFIG_CHILDREN,
 						       i);
 
-	  get_bootpath (child, bootpath, devid);
+	  get_bootpath (child, bootpath, devid, inguid);
 
 	  grub_free (child);
 
@@ -340,12 +342,16 @@
   char *fsname;
   char *bootfs;
   char *poolname;
-  grub_uint64_t mdnobj;
+  char def_path[512];
+  struct grub_zfs_data * data;
+  grub_uint64_t mdnobj, guid;
+  int def = 0;
 
-  if (argc < 1)
+  if (argc <  1) 
     return grub_error (GRUB_ERR_BAD_ARGUMENT, N_("one argument expected"));
 
   devname = grub_file_get_device_name (args[0]);
+
   if (grub_errno)
     return grub_errno;
 
@@ -362,8 +368,14 @@
   else
     fsname = args[0];
 
-  if (!err)
+  if (grub_strcmp(fsname, "default") == 0)
+    ++def;
+
+  if (! def) {
     err = grub_zfs_getmdnobj (dev, fsname, &mdnobj);
+  } else {
+    err = get_default_bootfs_obj(dev, def_path, &mdnobj);
+  }
 
   grub_device_close (dev);
 
@@ -379,9 +391,10 @@
     }
 
   nv = grub_zfs_nvlist_lookup_nvlist (nvlist, ZPOOL_CONFIG_VDEV_TREE);
+  grub_zfs_nvlist_lookup_uint64 (nvlist, "guid", &guid);
 
-  if (nv)
-    get_bootpath (nv, &bootpath, &devid);
+  if (nv && guid)
+    get_bootpath (nv, &bootpath, &devid, guid);
 
   grub_free (nv);
   grub_free (nvlist);
@@ -401,6 +414,11 @@
   else
     grub_printf ("%s\n", bootfs);
 
+  if ((def) && (argc >= 3))
+    grub_env_set(args[2], def_path);
+  else if (def)
+    grub_printf("%s\n", def_path);
+
   grub_free (bootfs);
   grub_free (poolname);
   grub_free (bootpath);
